﻿// Node class, Aaron Steed 2007
// Based on Carden / Steed algorithm
class Node {
	var x:Number;
	var y:Number;
	var g:Number = 0.0;
	var h:Number = 0.0;
	var f:Number = 0.0;
	var parent:Node;
	var n:Number;
	var walkable:Boolean;
	var death:Boolean;
	var links:Array;
	var assets:Array;
	function Node(x:Number, y:Number, n:Number) {
		this.x = x;
		this.y = y;
		this.n = n;
		links = new Array();
		assets = new Array();
		walkable = true;
		parent = null;
	}
	// Reset Node
	function reset():Void {
		h = f = g = 0.0;
		parent = null;
	}
	// Distance travelled calculation
	function setG():Void {
		g = parent.g + dist(parent);
	}
	// Heuristic distance to Node
	function setH(fin:Node):Void {
		h = dist(fin);
	}
	// Calculate F
	function setF(fin:Node):Void {
		setG();
		setH(fin);
		f = g + h;
	}
	// Distance to a Node
	function dist(n:Node):Number {
		var xd = x - n.x;
		var yd = y - n.y;
		return Math.sqrt((xd * xd) + (yd * yd));
	}
	// The index of a Node in this Node's links Array
	function indexOf(n:Node):Number {
		for (var i = 0; i < links.length; i++) {
			if (links[i] == n) {
				return i;
			}
		}
		return -1;
	}
	// Adds a Node to outgoing links
	function addLink(n:Node):Void {
		links.push(n);
	}
	// Returns false if Node n is not perpendicular to this Node
	function straightLink(n:Node):Boolean {
		var dimDelta = 0;
		if (x != n.x) {
			dimDelta++;
		}
		if (y != n.y) {
			dimDelta++;
		}
		if (dimDelta == 1) {
			return true;
		}
		return false;
	}
	// Disconnects all incoming links to this Node
	function disconnect():Void {
		for (var i = 0; i < links.length; i++) {
			var index = links[i].indexOf(this);
			if (index > -1) {
				links[i].links.splice(index, 1);
			}
		}
	}
	// Creates a square path around this Node
	function radialDisconnect():Void {
		var radius = 0.0;
		for (var i = 0; i < links.length; i++) {
			if (straightLink(links[i])) {
				radius = dist(links[i]);
				break;
			}
		}
		for (var i = 0; i < links.length; i++) {
			var removeMe:Array = new Array();
			for (var j = 0; j < links[i].links.length; j++) {
				var midX = (links[i].x + links[i].links[j].x) * 0.5;
				var midY = (links[i].y + links[i].links[j].y) * 0.5;
				if (dist(new Node(midX, midY, -1)) <= radius) {
					removeMe.push(links[i].links[j]);
				}
			}
			for (var j = 0; j < removeMe.length; j++) {
				var index = links[i].indexOf(removeMe[j]);
				if (index > -1) {
					links[i].links.splice(index, 1);
				}
			}
		}
	}
}
